<?php

/**
 * Cloudrexx
 *
 * @link      http://www.cloudrexx.com
 * @copyright Cloudrexx AG 2007-2015
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Cloudrexx" is a registered trademark of Cloudrexx AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */

/**
 * Wysiwyg
 *
 * @copyright   CLOUDREXX CMS - CLOUDREXX AG
 * @author      CLOUDREXX Development Team <info@cloudrexx.com>
 * @package     cloudrexx
 * @subpackage  core_wysiwyg
 */

namespace Cx\Core\Wysiwyg;
use Cx\Core_Modules\MediaBrowser\Model\Entity\MediaBrowser;

/**
 * Class WysiwygException
 *
 * @copyright   Cloudrexx AG
 * @author      Manuel Schenk <manuel.schenk@comvation.com>
 * @version     1.0.0
 * @package     cloudrexx
 * @subpackage  core_wysiwyg
 */
class WysiwygException extends \Exception {}

/**
 * Wysiqyg class
 *
 * @copyright   CLOUDREXX CMS - CLOUDREXX AG
 * @author      Thomas Daeppen <thomas.daeppen@comvation.com>
 * @author      Michael Räss <michael.raess@comvation.com>
 * @author      Ueli Kramer <ueli.kramer@comvation.com>
 * @version     3.0.0
 * @package     cloudrexx
 * @subpackage  core_wysiwyg
 */

class Wysiwyg extends \Cx\Model\Base\EntityBase
{
    /**
     * options for the different types of wysiwyg editors
     * @var array the types which are available for cloudrexx wysiwyg editors
     */
    protected $types = array(
        'small' => array(
            'toolbar' => 'Small',
            'width' => '100%',
            'height' => 200,
            'fullPage' => 'false',
            'extraPlugins' => array(),
            'removePlugins' => array('bbcode'),
        ),
        'full' => array(
            'toolbar' => 'Full',
            'width' => '100%',
            'height' => 450,
            'fullPage' => 'false',
            'extraPlugins' => array(),
            'removePlugins' => array('bbcode'),
        ),
        'fullpage' => array(
            'toolbar' => 'Full',
            'width' => '100%',
            'height' => 450,
            'fullPage' => 'true',
            'extraPlugins' => array(),
            'removePlugins' => array('bbcode'),
        ),
        'bbcode' => array(
            'toolbar' => 'BBCode',
            'width' => '100%',
            'height' => 200,
            'fullPage' => 'false',
            'extraPlugins' => array(),
        ),
    );

    /**
     * @var string the value for the textarea html attribute "name"
     */
    protected $name;

    /**
     * @var string the value for the textarea html attribute "value"
     */
    protected $value;

    /**
     * @var string the type of wysiwyg editor
     */
    protected $type;

    /**
     * @var int the language id of current language
     */
    protected $langId;

    /**
     * @var array array of extra plugins added for the wysiwyg editor
     */
    protected $extraPlugins;

    /**
     * @var array array of plugins to be deactivated in the wysiwyg editor
     */
    protected $removePlugins;

    /**
     * The SystemComponentController that did instanciate this Wysiwyg instance.
     *
     * @var \Cx\Core\Core\Model\Entity\SystemComponentController SystemComponentController that instanciated this Wysiwyg instance
     */
    protected $callingSystemComponentController;

     /**
      * Initialize WYSIWYG editor
      * If $systemComponentController is not specified, the call trace is searched for the component calling this constructor.
      * @param string $name the name content for name attribute
      * @param string $value (optional) content for value attribute
      * @param string $type (optional) the type of editor to use: possible types are "small", "full", "bbcode", default is "small"
      * @param null|int $langId (optional) the language id, by default FRONTEND_LANG_ID is used
      * @param array $extraPlugins (optional) extra plugins to activate
      * @param array $removePlugins (optional) deactivate plugins
      * @param \Cx\Core\Core\Model\Entity\SystemComponentController $systemComponentController (optional) ComponentController of the component that uses the object generated by this constructor
     * @throws WysiwygException If no $systemComponentController is provided and none can be found
     */
    public function __construct($name, $value = '', $type = 'small', $langId = null, $extraPlugins = array(), $removePlugins = array(), \Cx\Core\Core\Model\Entity\SystemComponentController $systemComponentController = null)
    {
        // Sets provided SystemComponentController that instanciated this Wysiwyg instance
        $this->callingSystemComponentController = $systemComponentController;
        if (!$this->callingSystemComponentController) {
            // Searches the calling SystemComponentController intelligently by RegEx on backtrace stack frame
            $traces = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
            $trace = end($traces);
            if (empty($trace['class'])) {
                throw new WysiwygException('No SystemComponentController found that instanciated Wysiwyg');
            }
            $matches = array();
            preg_match(
                '/Cx\\\\(?:Core|Core_Modules|Modules)\\\\([^\\\\]*)\\\\/',
                $trace['class'],
                $matches
            );
            $this->callingSystemComponentController = $this->cx->getComponent($matches[1]);
        }

        $this->name = $name;
        $this->value = $value;
        $this->type = strtolower($type);
        $this->langId = $langId ? intval($langId) : FRONTEND_LANG_ID;
        $this->extraPlugins = $extraPlugins;
        $this->removePlugins = $removePlugins;
    }

    /**
     * Get the html source code for the wysiwyg editor
     *
     * @return string
     */
    public function getSourceCode()
    {
        $this->initJsVariablesForUploader();

        $mediaBrowserCkeditor = new MediaBrowser($this->callingSystemComponentController);
        $mediaBrowserCkeditor->setCallback('ckeditor_image_callback');
        $mediaBrowserCkeditor->setOptions(
            array(
                'type'           => 'button',
                'style'          => 'display:none',
                'id'             => 'ckeditor_image_button',
                'startmediatype' => $this->getMediaSource()->getName()
            )
        );

        \JS::activate('ckeditor');
        \JS::activate('jquery');

        $configPath = $this->getComponentController()->getConfigPath();
        $options = array(
            "customConfig: CKEDITOR.getUrl('".$configPath."')",
            "width: '" . $this->types[$this->type]['width'] . "'",
            "height: '" . $this->types[$this->type]['height'] . "'",
            "toolbar: '" . $this->types[$this->type]['toolbar'] . "'",
            "fullPage: " . $this->types[$this->type]['fullPage']
        );

        $extraPlugins = array_merge($this->extraPlugins, $this->types[$this->type]['extraPlugins']);
        if (!empty($extraPlugins)) {
            $options[] = "extraPlugins: '" . implode(',', $extraPlugins) . "'";
        }

        $removePlugins = array_merge($this->removePlugins, $this->types[$this->type]['removePlugins']);
        if (!empty($removePlugins)) {
            $options[] = "removePlugins: '" . implode(',', $removePlugins) . "'";
        }

        $onReady = "CKEDITOR.replace('".$this->name."', { %s });";
        \JS::registerCode('
            $J(function(){
                '.sprintf($onReady, implode(",\r\n", $options)).'
            });
        ');

        return $mediaBrowserCkeditor->getXHtml('mediabrowser').'<textarea name="'.$this->name.'" style="width: 100%; height: ' . $this->types[$this->type]['height'] . 'px">'.$this->value.'</textarea>';
    }

    /**
     * Get MediaSource based on the component
     *
     * @return Cx\Core\MediaSource\Model\Entity\MediaSource
     */
    public function getMediaSource()
    {
        $mediaSourceManager = $this->cx->getMediaSourceManager();
        $mediaSource = $mediaSourceManager
            ->getMediaSourceByComponent($this->callingSystemComponentController);

        if ($mediaSource) {
            return $mediaSource;
        }

        //If MediaSource does not exists, set the first MediaSource from the MediaSources list
        return current($mediaSourceManager->getMediaTypes());
    }

    /**
     * Initialize the uploader and set uploader id and target path as javascript variable
     */
    protected function initJsVariablesForUploader()
    {
        $uploader       = new \Cx\Core_Modules\Uploader\Model\Entity\Uploader();
        $mediaSourceDir = $this->getMediaSource()->getDirectory();
        $cxJs           = \ContrexxJavascript::getInstance();
        $cxJs->setVariable(array(
            'ckeditorUploaderId'   => $uploader->getId(),
            'ckeditorUploaderPath' => $mediaSourceDir[1] . '/'
        ), 'wysiwyg');
    }

    /**
     * Get safe BBCode
     *
     * @param string $bbcode the unsafe BBCode
     * @param bool $html return as html code
     * @return string
     */
    public static function prepareBBCodeForDb($bbcode, $html = false)
    {
        $bbcode = strip_tags($bbcode);
        if ($html) {
            $bbcode = self::prepareBBCodeForOutput($bbcode);
        }
        return contrexx_input2db($bbcode);
    }

    /**
     * Convert BBCode to HTML
     *
     * This code comes from the forum module, feel free to rewrite
     *
     * @param string $bbcode the BBCode which should be a html output
     * @return string the xhtml output
     */
    public static function prepareBBCodeForOutput($bbcode)
    {
        $BBCodeHandler = new \Cx\Core\Wysiwyg\BBCodeHandler();
        return $BBCodeHandler->parse($bbcode);
    }

    /**
     * Alias for the method getSourceCode()
     *
     * @return string
     */
    public function __toString()
    {
        return $this->getSourceCode();
    }

    /**
     * @param int $langId
     */
    public function setLangId($langId)
    {
        $this->langId = $langId;
    }

    /**
     * @return int
     */
    public function getLangId()
    {
        return $this->langId;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = strtolower($type);
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $value
     */
    public function setValue($value)
    {
        $this->value = $value;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param array $extraPlugins
     */
    public function setExtraPlugins($extraPlugins)
    {
        $this->extraPlugins = $extraPlugins;
    }

    /**
     * @return array
     */
    public function getExtraPlugins()
    {
        return $this->extraPlugins;
    }

    /**
     * Set a list of plugins that shall be deactivated
     *
     * @param array $removePlugins List of plugin names
     */
    public function setRemovePlugins($removePlugins)
    {
        $this->removePlugins = $removePlugins;
    }

    /**
     * Fetch the list of deactivated plugins
     *
     * @return array List of plugin names
     */
    public function getRemovePlugins()
    {
        return $this->removePlugins;
    }
}
